<?

/*
 * General class to hold non-photo objects
 */

class item
{
  var $filename, $type;
  var $name, $date, $comment, $keywords, $size;
  var $nodes;

  function item ()
    {
      $this->reset_item ();
    }

  function reset_item ()
    {
      $this->type = "item";
      $this->filename = false;
      $this->size = false;
      $this->nodes = array();
      $this->name = $this->date = $this->comment = false;
    }

  function set_file ($name)
    {
      global $config;
      
      $this->reset_item ();
      $this->filename = $name;

      $config->load_file ($this, $name);
    }
}


/*
 * General class to hold photos
 */

class photo extends item
{
  var $rel_cachedir, $cachedir, $basename;
  var $sizex, $sizey, $rotate;

  function photo ()
    {
      $this->reset_photo ();
    }

  function reset_photo ()
    {
      global $config;

      $this->reset_item ();

      $this->type = "photo";
      $this->basename = false;
      $this->cachedir = $config->get ("general", "cachedir");
      $this->rel_cachedir = $config->get ("general", "rel_cachedir");
      $this->rotate = $this->sizex = $this->sizey = false;
    }
  
  function set_photo ($name)
    {
      global $config;
      
      $this->reset_photo ();
      $this->filename = $name;

      $basename = basename ($this->filename);
      $pos = strrpos ($basename, ".");
      $this->basename = substr ($basename, 0, $pos);

      $config->load_photo ($this, $name);
    }

  /* Generate a thumbnail or scaled image, and cache it */
  function resize ($width)
    {
      global $config;
      
      /* Compute absolute and relative name of the target */
      $cached = "/" . $this->basename . "-" . $width . ".jpg";
      $absolute = $this->cachedir . $cached;
      $result = $this->rel_cachedir . $cached;

      if (!file_exists ($absolute)) 
	{
	  /* Build parameter list to convert */
	  $param = " -quality ".$config->get ("photos", "quality");
	  if ($width != $this->sizex)
	    $param .= " -resize $width";
	  if ($this->rotate)
	    $param .= " -rotate ".($this->rotate * 90);
	  /* And generate it */
	  exec ("convert ".$this->filename." ".$param." ".$absolute);
	}
      return $result;
    }
}

function item_cmp (&$a, &$b)
{
  return strcmp ($a->name, $b->name);
}

/* 
 * Class holding a node in the main tree
 */

class node
{
  var $items, $childs, $fathers;
  var $excludes;
  var $name, $login, $pass, $path, $rel_path;
  
  function node()
    {
      $this->reset_node ();
    }

  function reset_node ()
    {
      global $config;
      
      $this->items = $this->childs = $this->fathers = array ();
      $this->login = $this->pass = array ();
      $this->excludes = explode (" ", $config->get ("tree", "excludes"));
      $this->path = $this->rel_path = $this->name = false;
    }
  
  function use_dir ($path, $rel_path)
    {
      global $config;

      $this->reset_node ();
      $this->path = $path;
      $this->rel_path = $rel_path;
 
      $config->load_node ($this, $path);
      
      $dir = opendir ($path);
      while (($file = readdir ($dir)) !== false)
	{
	  /* Ignores some files */
	  if (substr ($file, 0, 1) == '.')
	    continue;
	  $found = false;
	  for ($i = 0; $i < count ($this->excludes); $i++)
	    if (ereg ($this->excludes[$i], $file))
	      $found = true;
	  if ($found)
	    continue;

	  $fn = $path . '/' . $file;

	  /* Sub node ? */
	  if (is_dir ($fn))
	    {
	      $node = new node;
	      $node->use_dir($fn, $rel_path . '/' . $file);
	      $node->father[] = $this;
	      $this->childs[] = $node;
	      continue;
	    }
	}
      closedir ($dir);
    }  

  function load_items ()
    {
      global $config;

      $dir = opendir ($this->path);
      while (($file = readdir ($dir)) !== false)
	{
	  /* Ignores some files */
	  if (substr ($file, 0, 1) == '.')
	    continue;
	  $found = false;
	  for ($i = 0; $i < count ($this->excludes); $i++)
	    if (ereg ($this->excludes[$i], $file))
	      $found = true;
	  if ($found)
	    continue;

	  $fn = $this->path . '/' . $file;

	  /* Sub node ? */
	  if (is_dir ($fn))
	    continue;

	  /* Real file */
	  $tpath = realpath ($fn);
	  $ext = substr ($tpath, strrpos ($tpath, "."));	  
	  if (stristr ($config->get ("photos", "extensions"), $ext))
	    {
	      $photo = new photo;
	      $photo->set_photo ($tpath);
	      $item =& $photo;
	    }
	  else
	    {
	      $item = new item;
	      $item->set_file ($tpath);
	    }
	  $this->items[] = $item;
	  $item->nodes[] = $this;
	}
      usort ($this->items, "item_cmp");
      closedir ($dir);
    }  

}

/*
 * Class holding configuration
 */

class config 
{
  var $values;
  var $backends;
  var $langs;

  function config ()
    {
      $this->values = array ();
      $this->backends = array ();
      $this->langs = array();
    }
  
  function get ($field, $name)
    {
      return $this->values[$field][$name];
    }

  function add_lang($name, &$handler)
    {
      $this->langs[$name] = $handler;
    }

  function load ($filename)
    {
      include ($filename);

      while (list ($idx, $name) = each ($backend_names))
	{
	  include ($name);
	  $this->backends[$backend_name] = new $backend_classname;
	}
    }

  function load_file (&$item, $file)
    {
      reset ($this->backends);
      while (list ($name, $back) = each ($this->backends))
	$back->load_file ($item, $file);
      reset ($this->backends);
      while (list ($name, $back) = each ($this->backends))
	$back->postload_photo ($item, $file);
    }
  
  function load_photo (&$item, $file)
    {
      reset ($this->backends);
      while (list ($name, $back) = each ($this->backends))
	$back->load_photo ($item, $file);
      reset ($this->backends);
      while (list ($name, $back) = each ($this->backends))
	$back->postload_photo ($item, $file);
    }

  function load_node (&$item, $file)
    {
      reset ($this->backends);
      while (list ($name, $back) = each ($this->backends))
	$back->load_node ($item, $file);
      reset ($this->backends);
      while (list ($name, $back) = each ($this->backends))
	$back->postload_node ($item, $file);
    }

  function translate($str)
    {
      $lang = $this->get ("general", "lang");
      if (!$lang)
	return $str;
      $handler = $this->langs[$lang];
      if (!$handler)
	return $str;
      return $handler->translate ($str, $lang);
    }
}

?>
